@@ -141,6 +141,8 @@ struct obs_frontend_callbacks {
 	virtual void obs_frontend_open_source_filters(obs_source_t *source) = 0;
 	virtual void
 	obs_frontend_open_source_interaction(obs_source_t *source) = 0;
+	virtual void
+	obs_frontend_open_sceneitem_edit_transform(obs_sceneitem_t *item) = 0;
 
 	virtual char *obs_frontend_get_current_record_output_path(void) = 0;
 	virtual const char *
@@ -151,6 +153,13 @@ struct obs_frontend_callbacks {
 	virtual char *obs_frontend_get_last_recording(void) = 0;
 	virtual char *obs_frontend_get_last_screenshot(void) = 0;
 	virtual char *obs_frontend_get_last_replay(void) = 0;
+
+	virtual void obs_frontend_add_undo_redo_action(const char *name,
+						       const undo_redo_cb undo,
+						       const undo_redo_cb redo,
+						       const char *undo_data,
+						       const char *redo_data,
+						       bool repeatable) = 0;
 };
 
 EXPORT void
