@@ -40,7 +40,7 @@ void OBSBasicTransform::HookWidget(QWidget *widget, const char *signal,
 #define ISCROLL_CHANGED SIGNAL(valueChanged(int))
 #define DSCROLL_CHANGED SIGNAL(valueChanged(double))
 
-OBSBasicTransform::OBSBasicTransform(OBSBasic *parent)
+OBSBasicTransform::OBSBasicTransform(OBSSceneItem item, OBSBasic *parent)
 	: QDialog(parent), ui(new Ui::OBSBasicTransform), main(parent)
 {
 	setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
@@ -65,11 +65,11 @@ OBSBasicTransform::OBSBasicTransform(OBSBasic *parent)
 	ui->buttonBox->button(QDialogButtonBox::Close)->setDefault(true);
 
 	connect(ui->buttonBox->button(QDialogButtonBox::Reset),
-		SIGNAL(clicked()), this, SLOT(on_resetButton_clicked()));
+		&QPushButton::clicked, main,
+		&OBSBasic::on_actionResetTransform_triggered);
 
 	installEventFilter(CreateShortcutFilter());
 
-	OBSSceneItem item = FindASelectedItem(main->GetCurrentScene());
 	OBSScene scene = obs_sceneitem_get_scene(item);
 	SetScene(scene);
 	SetItem(item);
@@ -144,7 +144,9 @@ void OBSBasicTransform::SetItemQt(OBSSceneItem newItem)
 	if (item)
 		RefreshControls();
 
-	setEnabled(!!item);
+	bool enable = !!item;
+	ui->container->setEnabled(enable);
+	ui->buttonBox->button(QDialogButtonBox::Reset)->setEnabled(enable);
 }
 
 void OBSBasicTransform::OBSChannelChanged(void *param, calldata_t *data)
@@ -244,8 +246,10 @@ void OBSBasicTransform::RefreshControls()
 	obs_sceneitem_get_crop(item, &crop);
 
 	obs_source_t *source = obs_sceneitem_get_source(item);
-	float width = float(obs_source_get_width(source));
-	float height = float(obs_source_get_height(source));
+	uint32_t source_cx = obs_source_get_width(source);
+	uint32_t source_cy = obs_source_get_height(source);
+	float width = float(source_cx);
+	float height = float(source_cy);
 
 	int alignIndex = AlignToList(osi.alignment);
 	int boundsAlignIndex = AlignToList(osi.bounds_alignment);
@@ -258,6 +262,10 @@ void OBSBasicTransform::RefreshControls()
 	ui->sizeY->setValue(osi.scale.y * height);
 	ui->align->setCurrentIndex(alignIndex);
 
+	bool valid_size = source_cx != 0 && source_cy != 0;
+	ui->sizeX->setEnabled(valid_size);
+	ui->sizeY->setEnabled(valid_size);
+
 	ui->boundsType->setCurrentIndex(int(osi.bounds_type));
 	ui->boundsAlign->setCurrentIndex(boundsAlignIndex);
 	ui->boundsWidth->setValue(osi.bounds.x);
@@ -306,15 +314,23 @@ void OBSBasicTransform::OnControlChanged()
 		return;
 
 	obs_source_t *source = obs_sceneitem_get_source(item);
-	double width = double(obs_source_get_width(source));
-	double height = double(obs_source_get_height(source));
+	uint32_t source_cx = obs_source_get_width(source);
+	uint32_t source_cy = obs_source_get_height(source);
+	double width = double(source_cx);
+	double height = double(source_cy);
 
 	obs_transform_info oti;
+	obs_sceneitem_get_info(item, &oti);
+
+	/* do not scale a source if it has 0 width/height */
+	if (source_cx != 0 && source_cy != 0) {
+		oti.scale.x = float(ui->sizeX->value() / width);
+		oti.scale.y = float(ui->sizeY->value() / height);
+	}
+
 	oti.pos.x = float(ui->positionX->value());
 	oti.pos.y = float(ui->positionY->value());
 	oti.rot = float(ui->rotation->value());
-	oti.scale.x = float(ui->sizeX->value() / width);
-	oti.scale.y = float(ui->sizeY->value() / height);
 	oti.alignment = listToAlign[ui->align->currentIndex()];
 
 	oti.bounds_type = (obs_bounds_type)ui->boundsType->currentIndex();
@@ -343,11 +359,6 @@ void OBSBasicTransform::OnCropChanged()
 	ignoreTransformSignal = false;
 }
 
-void OBSBasicTransform::on_resetButton_clicked()
-{
-	main->on_actionResetTransform_triggered();
-}
-
 template<typename T> static T GetOBSRef(QListWidgetItem *item)
 {
 	return item->data(static_cast<int>(QtDataRole::OBSRef)).value<T>();
@@ -359,6 +370,6 @@ void OBSBasicTransform::OnSceneChanged(QListWidgetItem *current,
 	if (!current)
 		return;
 
-	obs_scene_t *scene = GetOBSRef<OBSScene>(current);
+	OBSScene scene = GetOBSRef<OBSScene>(current);
 	this->SetScene(scene);
 }
