@@ -17,7 +17,6 @@
 ******************************************************************************/
 
 #include "obs-scripting-python.h"
-#include "obs-scripting-config.h"
 #include <util/base.h>
 #include <util/platform.h>
 #include <util/darray.h>
@@ -1538,7 +1537,7 @@ static void python_tick(void *param, float seconds)
 	 * the cur_python_script state variable. Use the busy_script variable
 	 * to save and restore the value if not null.
 	 */
-	struct obs_python_script *busy_script;
+	struct obs_python_script *busy_script = NULL;
 	bool valid;
 	uint64_t ts = obs_get_video_frame_time();
 
@@ -1686,15 +1685,15 @@ bool obs_scripting_load_python(const char *python_path)
 
 #if RUNTIME_LINK
 	if (python_version.major == 3 && python_version.minor < 7) {
-#elif PY_VERSION_HEX < 0x030700b0
-	if (true) {
-#else
-	if (false) {
-#endif
 		PyEval_InitThreads();
 		if (!PyEval_ThreadsInitialized())
 			return false;
 	}
+#elif PY_VERSION_HEX < 0x03070000
+	PyEval_InitThreads();
+	if (!PyEval_ThreadsInitialized())
+		return false;
+#endif
 
 	/* ---------------------------------------------- */
 	/* Must set arguments for guis to work            */
@@ -1702,7 +1701,10 @@ bool obs_scripting_load_python(const char *python_path)
 	wchar_t *argv[] = {L"", NULL};
 	int argc = sizeof(argv) / sizeof(wchar_t *) - 1;
 
+	PRAGMA_WARN_PUSH
+	PRAGMA_WARN_DEPRECATION
 	PySys_SetArgv(argc, argv);
+	PRAGMA_WARN_POP
 
 #ifdef DEBUG_PYTHON_STARTUP
 	/* ---------------------------------------------- */
