@@ -735,7 +735,7 @@ void gs_shader_set_val(gs_sparam_t *param, const void *val, size_t size)
 	if (!count)
 		count = 1;
 
-	switch ((uint32_t)param->type) {
+	switch (param->type) {
 	case GS_SHADER_PARAM_FLOAT:
 		expected_size = sizeof(float);
 		break;
