@@ -68,19 +68,15 @@ char *find_libobs_data_file(const char *file)
 {
 	struct dstr path;
 
-	if (is_in_bundle()) {
-		NSBundle *frameworkBundle = [NSBundle
-			bundleWithIdentifier:@"com.obsproject.libobs"];
-		NSURL *bundleURL = [frameworkBundle bundleURL];
-		NSURL *libobsDataURL =
-			[bundleURL URLByAppendingPathComponent:@"Resources/"];
-		const char *libobsDataPath = [[libobsDataURL path]
-			cStringUsingEncoding:NSUTF8StringEncoding];
-		dstr_init_copy(&path, libobsDataPath);
-		dstr_cat(&path, "/");
-	} else {
-		dstr_init_copy(&path, OBS_INSTALL_DATA_PATH "/libobs/");
-	}
+	NSBundle *frameworkBundle =
+		[NSBundle bundleWithIdentifier:@"com.obsproject.libobs"];
+	NSURL *bundleURL = [frameworkBundle bundleURL];
+	NSURL *libobsDataURL =
+		[bundleURL URLByAppendingPathComponent:@"Resources/"];
+	const char *libobsDataPath = [[libobsDataURL path]
+		cStringUsingEncoding:NSUTF8StringEncoding];
+	dstr_init_copy(&path, libobsDataPath);
+	dstr_cat(&path, "/");
 
 	dstr_cat(&path, file);
 	return path.array;
@@ -491,6 +487,9 @@ static bool code_to_str(int code, struct dstr *str)
 
 void obs_key_to_str(obs_key_t key, struct dstr *str)
 {
+	const UniCharCount max_length = 16;
+	UniChar buffer[max_length];
+
 	if (localized_key_to_str(key, str))
 		return;
 
@@ -523,9 +522,7 @@ void obs_key_to_str(obs_key_t key, struct dstr *str)
 		goto err;
 	}
 
-	const UniCharCount max_length = 16;
 	UInt32 dead_key_state = 0;
-	UniChar buffer[max_length];
 	UniCharCount len = 0;
 
 	OSStatus err =
