@@ -52,7 +52,8 @@ static obs_service_t *obs_service_create_internal(const char *id,
 	service = bzalloc(sizeof(struct obs_service));
 
 	if (!obs_context_data_init(&service->context, OBS_OBJ_TYPE_SERVICE,
-				   settings, name, hotkey_data, private)) {
+				   settings, name, NULL, hotkey_data,
+				   private)) {
 		bfree(service);
 		return NULL;
 	}
@@ -410,16 +411,6 @@ const char *obs_service_get_id(const obs_service_t *service)
 		       : NULL;
 }
 
-const char *obs_service_get_output_type(const obs_service_t *service)
-{
-	if (!obs_service_valid(service, "obs_service_get_output_type"))
-		return NULL;
-
-	if (service->info.get_output_type)
-		return service->info.get_output_type(service->context.data);
-	return NULL;
-}
-
 void obs_service_get_supported_resolutions(
 	const obs_service_t *service,
 	struct obs_service_resolution **resolutions, size_t *count)
@@ -476,3 +467,58 @@ obs_service_get_supported_video_codecs(const obs_service_t *service)
 			service->context.data);
 	return NULL;
 }
+
+const char **
+obs_service_get_supported_audio_codecs(const obs_service_t *service)
+{
+	if (service->info.get_supported_audio_codecs)
+		return service->info.get_supported_audio_codecs(
+			service->context.data);
+	return NULL;
+}
+
+const char *obs_service_get_protocol(const obs_service_t *service)
+{
+	if (!obs_service_valid(service, "obs_service_get_protocol"))
+		return NULL;
+
+	return service->info.get_protocol(service->context.data);
+}
+
+/* OBS_DEPRECATED */
+const char *obs_service_get_output_type(const obs_service_t *service)
+{
+	return obs_service_get_preferred_output_type(service);
+}
+
+const char *obs_service_get_preferred_output_type(const obs_service_t *service)
+{
+	if (!obs_service_valid(service,
+			       "obs_service_get_preferred_output_type"))
+		return NULL;
+
+	if (service->info.get_output_type)
+		return service->info.get_output_type(service->context.data);
+	return NULL;
+}
+
+const char *obs_service_get_connect_info(const obs_service_t *service,
+					 uint32_t type)
+{
+	if (!obs_service_valid(service, "obs_service_get_info"))
+		return NULL;
+
+	if (!service->info.get_connect_info)
+		return NULL;
+	return service->info.get_connect_info(service->context.data, type);
+}
+
+bool obs_service_can_try_to_connect(const obs_service_t *service)
+{
+	if (!obs_service_valid(service, "obs_service_can_connect"))
+		return false;
+
+	if (!service->info.can_try_to_connect)
+		return true;
+	return service->info.can_try_to_connect(service->context.data);
+}
