@@ -45,35 +45,42 @@ static uint64_t tick_sources(uint64_t cur_time, uint64_t last_time)
 	/* ------------------------------------- */
 	/* call tick callbacks                   */
 
-	pthread_mutex_lock(&obs->data.draw_callbacks_mutex);
+	pthread_mutex_lock(&data->draw_callbacks_mutex);
 
-	for (size_t i = obs->data.tick_callbacks.num; i > 0; i--) {
+	for (size_t i = data->tick_callbacks.num; i > 0; i--) {
 		struct tick_callback *callback;
-		callback = obs->data.tick_callbacks.array + (i - 1);
+		callback = data->tick_callbacks.array + (i - 1);
 		callback->tick(callback->param, seconds);
 	}
 
-	pthread_mutex_unlock(&obs->data.draw_callbacks_mutex);
+	pthread_mutex_unlock(&data->draw_callbacks_mutex);
+
+	/* ------------------------------------- */
+	/* get an array of all sources to tick   */
+
+	da_clear(data->sources_to_tick);
+
+	pthread_mutex_lock(&data->sources_mutex);
+
+	source = data->sources;
+	while (source) {
+		obs_source_t *s = obs_source_get_ref(source);
+		if (s)
+			da_push_back(data->sources_to_tick, &s);
+		source = (struct obs_source *)source->context.hh_uuid.next;
+	}
+
+	pthread_mutex_unlock(&data->sources_mutex);
 
 	/* ------------------------------------- */
 	/* call the tick function of each source */
 
-	pthread_mutex_lock(&data->sources_mutex);
-
-	source = data->first_source;
-	while (source) {
-		obs_source_t *s = obs_source_get_ref(source);
-
-		if (s) {
-			obs_source_video_tick(s, seconds);
-			obs_source_release(s);
-		}
-
-		source = (struct obs_source *)source->context.next;
+	for (size_t i = 0; i < data->sources_to_tick.num; i++) {
+		obs_source_t *s = data->sources_to_tick.array[i];
+		obs_source_video_tick(s, seconds);
+		obs_source_release(s);
 	}
 
-	pthread_mutex_unlock(&data->sources_mutex);
-
 	return cur_time;
 }
 
@@ -144,9 +151,8 @@ static inline void render_main_texture(struct obs_core_video_mix *video)
 	pthread_mutex_lock(&obs->data.draw_callbacks_mutex);
 
 	for (size_t i = obs->data.draw_callbacks.num; i > 0; i--) {
-		struct draw_callback *callback;
-		callback = obs->data.draw_callbacks.array + (i - 1);
-
+		struct draw_callback *const callback =
+			obs->data.draw_callbacks.array + (i - 1);
 		callback->draw(callback->param, base_width, base_height);
 	}
 
@@ -156,6 +162,16 @@ static inline void render_main_texture(struct obs_core_video_mix *video)
 
 	video->texture_rendered = true;
 
+	pthread_mutex_lock(&obs->data.draw_callbacks_mutex);
+
+	for (size_t i = 0; i < obs->data.rendered_callbacks.num; ++i) {
+		struct rendered_callback *const callback =
+			&obs->data.rendered_callbacks.array[i];
+		callback->rendered(callback->param);
+	}
+
+	pthread_mutex_unlock(&obs->data.draw_callbacks_mutex);
+
 	GS_DEBUG_MARKER_END();
 	profile_end(render_main_texture_name);
 }
@@ -221,27 +237,19 @@ static const char *render_output_texture_name = "render_output_texture";
 static inline gs_texture_t *
 render_output_texture(struct obs_core_video_mix *mix)
 {
-	struct obs_core_video *video = &obs->video;
+	struct obs_video_info *const ovi = &mix->ovi;
 	gs_texture_t *texture = mix->render_texture;
 	gs_texture_t *target = mix->output_texture;
-	uint32_t width = gs_texture_get_width(target);
-	uint32_t height = gs_texture_get_height(target);
-
-	gs_effect_t *effect = get_scale_effect(mix, width, height);
-	gs_technique_t *tech;
-
-	if (video_output_get_format(mix->video) == VIDEO_FORMAT_BGRA) {
-		tech = gs_effect_get_technique(effect, "DrawAlphaDivide");
-	} else {
-		if ((width == mix->ovi.base_width) &&
-		    (height == mix->ovi.base_height))
-			return texture;
-
-		tech = gs_effect_get_technique(effect, "Draw");
-	}
+	const uint32_t width = gs_texture_get_width(target);
+	const uint32_t height = gs_texture_get_height(target);
+	if ((width == ovi->base_width) && (height == ovi->base_height))
+		return texture;
 
 	profile_start(render_output_texture_name);
 
+	gs_effect_t *effect = get_scale_effect(mix, width, height);
+	gs_technique_t *tech = gs_effect_get_technique(effect, "Draw");
+
 	gs_eparam_t *image = gs_effect_get_param_by_name(effect, "image");
 	gs_eparam_t *bres =
 		gs_effect_get_param_by_name(effect, "base_dimension");
@@ -391,6 +399,7 @@ static const char *stage_output_texture_name = "stage_output_texture";
 static inline void
 stage_output_texture(struct obs_core_video_mix *video, int cur_texture,
 		     gs_texture_t *const *const convert_textures,
+		     gs_texture_t *output_texture,
 		     gs_stagesurf_t *const *const copy_surfaces,
 		     size_t channel_count)
 {
@@ -401,7 +410,7 @@ stage_output_texture(struct obs_core_video_mix *video, int cur_texture,
 	if (!video->gpu_conversion) {
 		gs_stagesurf_t *copy = copy_surfaces[0];
 		if (copy)
-			gs_stage_texture(copy, video->output_texture);
+			gs_stage_texture(copy, output_texture);
 		video->active_copy_surfaces[cur_texture][0] = copy;
 
 		for (size_t i = 1; i < NUM_CHANNELS; ++i)
@@ -536,7 +545,7 @@ static inline void render_video(struct obs_core_video_mix *video,
 		gs_stagesurf_t *const *copy_surfaces =
 			video->copy_surfaces[cur_texture];
 		size_t channel_count = NUM_CHANNELS;
-		gs_texture_t *texture = render_output_texture(video);
+		gs_texture_t *output_texture = render_output_texture(video);
 
 #ifdef _WIN32
 		if (gpu_active) {
@@ -547,9 +556,10 @@ static inline void render_video(struct obs_core_video_mix *video,
 		}
 #endif
 
-		if (video->gpu_conversion)
+		if (video->gpu_conversion) {
 			render_convert_texture(video, convert_textures,
-					       texture);
+					       output_texture);
+		}
 
 #ifdef _WIN32
 		if (gpu_active) {
@@ -558,10 +568,11 @@ static inline void render_video(struct obs_core_video_mix *video,
 		}
 #endif
 
-		if (raw_active)
+		if (raw_active) {
 			stage_output_texture(video, cur_texture,
-					     convert_textures, copy_surfaces,
-					     channel_count);
+					     convert_textures, output_texture,
+					     copy_surfaces, channel_count);
+		}
 	}
 
 	gs_set_render_target(NULL, NULL);
@@ -730,6 +741,33 @@ static void set_gpu_converted_data(struct video_frame *output,
 
 		break;
 	}
+	case VIDEO_FORMAT_P216: {
+		const uint32_t width_x2 = info->width * 2;
+		const uint32_t height = info->height;
+
+		set_gpu_converted_plane(width_x2, height, input->linesize[0],
+					output->linesize[0], input->data[0],
+					output->data[0]);
+
+		set_gpu_converted_plane(width_x2, height, input->linesize[1],
+					output->linesize[1], input->data[1],
+					output->data[1]);
+
+		break;
+	}
+	case VIDEO_FORMAT_P416: {
+		const uint32_t height = info->height;
+
+		set_gpu_converted_plane(info->width * 2, height,
+					input->linesize[0], output->linesize[0],
+					input->data[0], output->data[0]);
+
+		set_gpu_converted_plane(info->width * 4, height,
+					input->linesize[1], output->linesize[1],
+					input->data[1], output->data[1]);
+
+		break;
+	}
 
 	case VIDEO_FORMAT_NONE:
 	case VIDEO_FORMAT_YVYU:
@@ -748,6 +786,7 @@ static void set_gpu_converted_data(struct video_frame *output,
 	case VIDEO_FORMAT_YUVA:
 	case VIDEO_FORMAT_YA2L:
 	case VIDEO_FORMAT_AYUV:
+	case VIDEO_FORMAT_V210:
 		/* unimplemented */
 		;
 	}
@@ -1055,7 +1094,6 @@ static inline void update_active_state(struct obs_core_video_mix *video)
 		os_atomic_load_long(&video->gpu_encoder_active) > 0;
 	const bool active = raw_active || gpu_active;
 #else
-	const bool gpu_active = 0;
 	const bool active = raw_active;
 #endif
 
