@@ -20,7 +20,7 @@
 
 #if defined(_WIN32) && LIBCURL_VERSION_NUM >= 0x072c00
 
-#ifdef CURLSSLOPT_REMOVE_BEST_EFFORT
+#ifdef CURLSSLOPT_REVOKE_BEST_EFFORT
 #define CURL_OBS_REVOKE_SETTING CURLSSLOPT_REVOKE_BEST_EFFORT
 #else
 #define CURL_OBS_REVOKE_SETTING CURLSSLOPT_NO_REVOKE
