@@ -126,8 +126,10 @@ bool ffmpeg_hls_mux_start(void *data)
 	service = obs_output_get_service(stream->output);
 	if (!service)
 		return false;
-	path_str = obs_service_get_url(service);
-	stream_key = obs_service_get_key(service);
+	path_str = obs_service_get_connect_info(
+		service, OBS_SERVICE_CONNECT_INFO_SERVER_URL);
+	stream_key = obs_service_get_connect_info(
+		service, OBS_SERVICE_CONNECT_INFO_STREAM_KEY);
 	dstr_copy(&stream->stream_key, stream_key);
 	dstr_copy(&path, path_str);
 	dstr_replace(&path, "{stream_key}", stream_key);
@@ -331,6 +333,7 @@ struct obs_output_info ffmpeg_hls_muxer = {
 	.id = "ffmpeg_hls_muxer",
 	.flags = OBS_OUTPUT_AV | OBS_OUTPUT_ENCODED | OBS_OUTPUT_MULTI_TRACK |
 		 OBS_OUTPUT_SERVICE,
+	.protocols = "HLS",
 #ifdef ENABLE_HEVC
 	.encoded_video_codecs = "h264;hevc",
 #else
